#ifndef MSP_STRINGS_FORMAT_H_
#define MSP_STRINGS_FORMAT_H_

#include <string>
#include "lexicalcast.h"

namespace Msp {

/**
Printf-like string formatter class.
*/
class Formatter
{
private:
	std::string fmt;
	std::string::iterator pos;
	std::string result;

public:
	Formatter(const std::string &);

	/** Extracts the next conversion from the format string and formats the
	given value with it.  Will throw if no more conversions are found. */
	template<typename T>
	Formatter &operator()(const T &a)
	{
		result += lexical_cast<std::string>(a, get_conversion());
		advance();
		return *this;
	}

	/** Returns the result of the formatting operation.  Will throw if not
	enough values have been fed to the formatter. */
	const std::string &str() const;

private:
	/** Advances the iterator to the next conversion, adding literal characters
	to the result.  The iterator is left at the second character of the
	conversion (i.e. after the %). */
	void advance();

	/** Reads the next conversion from the format string and returns a
	corresponding Fmt object. */
	Fmt get_conversion();
};

inline Formatter format(const std::string &f)
{ return Formatter(f); }

template<typename A1>
inline std::string format(const std::string &f, const A1 &a1)
{ return Formatter(f)(a1).str(); }

template<typename A1, typename A2>
inline std::string format(const std::string &f, const A1 &a1, const A2 &a2)
{ return Formatter(f)(a1)(a2).str(); }

template<typename A1, typename A2, typename A3>
inline std::string format(const std::string &f, const A1 &a1, const A2 &a2, const A3 &a3)
{ return Formatter(f)(a1)(a2)(a3).str(); }

template<typename A1, typename A2, typename A3, typename A4>
inline std::string format(const std::string &f, const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4)
{ return Formatter(f)(a1)(a2)(a3)(a4).str(); }

template<typename A1, typename A2, typename A3, typename A4, typename A5>
inline std::string format(const std::string &f, const A1 &a1, const A2 &a2, const A3 &a3, const A4 &a4, const A5 &a5)
{ return Formatter(f)(a1)(a2)(a3)(a4)(a5).str(); }

} // namespace Msp

#endif
